/***************************************************************************
                          klegacyconfig.cpp  -  description
                             -------------------
    begin                : Sat Jan 26 13:17:25 NZDT 2002
    copyright            : (C) 2002 by Jonathon Sim
    email                : jsim@free.net.nz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "klegacyconfig.h"

#include <klocale.h>
#include <kglobal.h>
#include <kwin.h>
#include <kdebug.h>
#include <stdlib.h>
#include <qfileinfo.h>
#include <kconfig.h>
#include <kaboutdata.h>
#include <kwinmodule.h>
#include <kprocess.h>
#include <qxembed.h>
#include <qlabel.h>
#include <qlayout.h>

//A set of #defines that simplifies debugging output handling
#ifndef DEBUG_AREA
	#define DEBUG_AREA 0	//The kde debug area
#endif
#ifndef DEBUG_HEADING
	#define DEBUG_HEADING  "KLegacyConfig::"    //A prefix before any other debug messages
#endif
#ifndef DEBUG_OUT
	#define DBG  kdDebug(DEBUG_AREA) << DEBUG_HEADING  //An abreviation
#endif


KLegacyConfig::KLegacyConfig(QWidget *parent, const char *name):KCModule(parent,name)
{
	mAboutData = new KAboutData( "klegacyconfig", I18N_NOOP("KLegacyConfig"),
					    "0.6.1", I18N_NOOP("KLegacyConfig is a  KDE Control Center module, that embeds non-kde configuration tools into the control center."),
							KAboutData::License_GPL,
					    "(c) 2002, Jonathon Sim", 0, 0, "jsim@free.net.nz");



//Setup a layout manager
	if (this->layout())
		this->layout()->setAutoAdd(true);
	else {
		QLayout * layout = new QVBoxLayout(this);
		layout->setAutoAdd(true);
		layout->setResizeMode(QLayout::FreeResize);
	}
	DBG << "constructor" << endl;
	setFocusPolicy(QWidget::StrongFocus );

	// Load the configuration
	DBG <<   "Load Settings : Config File: "<< QString(QString(name) + "rc" )<< endl;
	KConfig * configuration = new KConfig(QString(QString(name )+ "rc") );
	configuration->setGroup("LegacyApp");
	mStrictTitleMatch =  configuration->readBoolEntry("StrictTitleMatch", false);
	mChildTitle = configuration->readEntry("ChildTitle");
	mChildCmdLine = configuration->readEntry("ChildCmdLine");
	DBG <<   "Load Settings : StrictTitleMatch "<<mStrictTitleMatch<< endl;
	DBG <<   "Load Settings : Child Title: "<< mChildTitle << endl;
	DBG <<   "Load Settings : Child Command Line: "<<mChildCmdLine << endl;
	
	mWaitForChildLabel = new QLabel(this);
	mWaitForChildLabel->setText(i18n("Please wait while the configuration program loads"));
	mWaitForChildLabel->show();
	
	setButtons(0);
	
	//Now we load the external config program
	load();
DBG << "constructor complete" << endl;

};


KLegacyConfig::~KLegacyConfig() {
	if (mChildProcess->isRunning()) mChildProcess->kill();
	delete mAboutData;
		
}
/**here we start the external config program, and watch for
	new windows until one appears that matches the name specified
	in the config file*/
void KLegacyConfig::load() {
	DBG << "loading" << endl;
	mKWin = new KWinModule(this);
	//mKWin->doNotManage( mChildTitle );
	    // connect to KWM events to get notification if window appears
	 connect(mKWin,  SIGNAL( windowAdded( WId) ),
	  	this, SLOT( slotWindowAdded( WId ) ) );

	mChildProcess = new  KShellProcess;
	*mChildProcess << mChildCmdLine;
	connect(mChildProcess,  SIGNAL(processExited(KProcess *)),
		this, SLOT(slotChildExited(KProcess *) ) );
	mChildProcess->start();
	

}
//These simplify the code below a little
inline bool nameMatch(KWin::Info info, QString name) {
	return  (  ( info.visibleName == name) ||   ( info.name == name)    )  ;
}
inline bool nameContains( KWin::Info info, QString name) {
	return   ( (info.visibleName.contains(name) )  || (info.name.contains(name) ) ) ;
}
/**When any new window is created, this slot is called.
	We check to see if the window has a matching name.  If so,
	we embed it*/
void  KLegacyConfig::slotWindowAdded(WId win) {
DBG << "New Window:" << KWin::info( win ).visibleName << endl;
	if ( (mStrictTitleMatch && nameMatch(KWin::info( win ), mChildTitle) ) || (!mStrictTitleMatch && nameContains(KWin::info( win ), mChildTitle) ) ) {
		DBG << "Swallowing Window:" << KWin::info( win ).visibleName << endl;
		mWaitForChildLabel->hide();
		setFocus();
		mEmbedWidget = new QXEmbed( this );
		mEmbedWidget->embed(win);
		mEmbedWidget->initialize();		
		mEmbedWidget->show();
		this->setFocusProxy( mEmbedWidget );
		setFocus();
		mEmbedWidget->setFocus();
		disconnect( mKWin,  SIGNAL( windowAdded( WId ) ),
	  	this, SLOT( slotWindowAdded( WId ) ) );
	  	emit changed(true);
	  }
}


/** Handles the closing of the child window */
void  KLegacyConfig::slotChildExited(KProcess *){
	DBG << "Child Process Exited" << endl;
	emit changed(false);
}

/**Loads defaults.  We dont have any*/
void KLegacyConfig::defaults() {
	emit changed(true);
}

/**We cant save settings*/
void KLegacyConfig::save() {
	emit changed(true);
}

/**Returns which buttons to show (ie none)*/
int KLegacyConfig::buttons () {
	return  0;//KCModule::Cancel ;//|KCModule::Help;
}

void KLegacyConfig::configChanged() {
	emit changed(true);
}

QString KLegacyConfig::quickHelp() const
{
	return i18n("This is a non-kde configuration program");
}

const KAboutData* KLegacyConfig::aboutData() {
	return mAboutData;
}

// ------------------------------------------------------------------------

extern "C"
{
	
	KCModule *create_KLegacyConfig(QWidget *parent, const char *name){
		KGlobal::locale()->insertCatalogue("KLegacyConfig");
		QFileInfo modname(getenv("KDE_CONTROL_CENTER_ENV1"));
		DBG << "Factory creating " << modname.baseName()  << endl;
		return new KLegacyConfig(parent, modname.baseName());
	}
}

#include "klegacyconfig.moc"
